' Functional Dependency Application
' Written by David J. Stein, Esq., Kevin Hardy, and Craig White. See "License.html" for terms and conditions of use.
'
' Check3NFStatus - written by David J. Stein, Esq.
' This module checks a functional dependency set for third normal form.

Public Class Check3NFStatus
 Inherits System.Windows.Forms.Form

#Region "Variables and initialization code"

 Dim Attributes As ArrayList
 Dim FunctionalDependencies As ArrayList
 Private x As Integer
 Private y As Integer

 Private Sub Check3NFStatusStatus_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
  Me.Location = New Point(x, y)
  lstAttributes.BeginUpdate()
  ' Construct a list of non-prime attributes
  Dim k As New KeyFinder
  Dim NonPrimeAttributes As ArrayList = k.FindNonPrimeAttributes(Attributes, FunctionalDependencies)
  For Each Attribute As Integer In NonPrimeAttributes
   Dim TestAttributeSet As New ArrayList
   Dim TargetAttributeSet As New ArrayList
   TargetAttributeSet.Add(Attribute)
   Dim TransitiveDeterminants As New ArrayList
   CheckTransitiveDependency(Attributes, FunctionalDependencies, TestAttributeSet, TargetAttributeSet, TransitiveDeterminants)
   If TransitiveDeterminants.Count > 0 Then
    For Each TransitiveDeterminant As ArrayList In TransitiveDeterminants
     Dim InsertString As String = ""
     For Each TransitiveDeterminantAttribute As Integer In TransitiveDeterminant
      InsertString = InsertString + Attributes(TransitiveDeterminantAttribute)
     Next
     InsertString = InsertString + " transitively determines " + Attributes(Attribute) + "."
     lstAttributes.Items.Add(InsertString)
    Next
   End If
  Next
  If lstAttributes.Items.Count = 0 Then
   lstAttributes.Items.Add("This relation is 3NF.")
  Else
   lstAttributes.Items.Insert(0, "This relation is not 3NF. The following")
   lstAttributes.Items.Insert(1, "transitive dependencies exist:")
  End If
  lstAttributes.EndUpdate()
 End Sub

#Region " Windows Form Designer generated code "

 Public Sub New(ByRef Attributes As ArrayList, ByRef FunctionalDependencies As ArrayList, ByVal x As Integer, ByVal y As Integer)
  MyBase.New()
  InitializeComponent()
  Me.x = x
  Me.y = y
  Me.Attributes = Attributes
  Me.FunctionalDependencies = FunctionalDependencies
 End Sub

 'Form overrides dispose to clean up the component list.
 Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
  If disposing Then
   If Not (components Is Nothing) Then
    components.Dispose()
   End If
  End If
  MyBase.Dispose(disposing)
 End Sub

 'Required by the Windows Form Designer
 Private components As System.ComponentModel.IContainer

 'NOTE: The following procedure is required by the Windows Form Designer
 'It can be modified using the Windows Form Designer.  
 'Do not modify it using the code editor.
 Friend WithEvents btnOK As System.Windows.Forms.Button
 Friend WithEvents Label1 As System.Windows.Forms.Label
 Friend WithEvents lstAttributes As System.Windows.Forms.ListBox
 <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
  Dim resources As System.Resources.ResourceManager = New System.Resources.ResourceManager(GetType(Check3NFStatus))
  Me.btnOK = New System.Windows.Forms.Button
  Me.Label1 = New System.Windows.Forms.Label
  Me.lstAttributes = New System.Windows.Forms.ListBox
  Me.SuspendLayout()
  '
  'btnOK
  '
  Me.btnOK.Location = New System.Drawing.Point(128, 416)
  Me.btnOK.Name = "btnOK"
  Me.btnOK.TabIndex = 5
  Me.btnOK.Text = "OK"
  '
  'Label1
  '
  Me.Label1.Location = New System.Drawing.Point(49, 8)
  Me.Label1.Name = "Label1"
  Me.Label1.Size = New System.Drawing.Size(232, 24)
  Me.Label1.TabIndex = 3
  Me.Label1.Text = "3NF Status:"
  Me.Label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter
  '
  'lstAttributes
  '
  Me.lstAttributes.Location = New System.Drawing.Point(8, 40)
  Me.lstAttributes.Name = "lstAttributes"
  Me.lstAttributes.SelectionMode = System.Windows.Forms.SelectionMode.None
  Me.lstAttributes.Size = New System.Drawing.Size(312, 368)
  Me.lstAttributes.TabIndex = 6
  '
  'Check3NFStatus
  '
  Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
  Me.ClientSize = New System.Drawing.Size(330, 448)
  Me.Controls.Add(Me.lstAttributes)
  Me.Controls.Add(Me.btnOK)
  Me.Controls.Add(Me.Label1)
  Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle
  Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
  Me.MaximizeBox = False
  Me.MinimizeBox = False
  Me.Name = "Check3NFStatus"
  Me.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent
  Me.Text = "Check 3NF Status"
  Me.ResumeLayout(False)

 End Sub

#End Region

#End Region

#Region "Button-click event handlers"

 Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
  Close()
 End Sub

#End Region

#Region "Core functions"

 Public Function CheckTransitiveDependency(ByRef Attributes As ArrayList, ByRef FunctionalDependencies As ArrayList, ByRef TestAttributeSet As ArrayList, ByRef TargetAttributeSet As ArrayList, ByRef TransitiveDeterminants As ArrayList)
  Dim Lin As New LinClosure
  Dim ResultSet As ArrayList = Lin.ComputeLinClosure(Attributes, TestAttributeSet, FunctionalDependencies)
  If ResultSet.Count < Attributes.Count Then   ' If this is a key - don't process any further; if not, keep going
   Dim TransitiveDeterminant As Boolean = True
   For Each TargetAttribute As Integer In TargetAttributeSet
    Dim FoundTarget As Boolean = False
    For Each TransitiveDeterminantAttribute As Integer In ResultSet
     If TransitiveDeterminantAttribute = TargetAttribute Then
      FoundTarget = True
     End If
    Next
    If FoundTarget = False Then
     TransitiveDeterminant = False
    End If
   Next
   If TransitiveDeterminant = True Then
    TransitiveDeterminants.Add(TestAttributeSet)
   End If
   ' Construct new array lists and try each of them
   Dim Start As Integer = 0
   If TestAttributeSet.Count > 0 Then
    Start = TestAttributeSet(TestAttributeSet.Count - 1) + 1
   End If
   For i As Integer = Start To Attributes.Count - 1
    Dim IsTargetAttribute As Boolean = False
    For Each TargetAttribute As Integer In TargetAttributeSet
     If i = TargetAttribute Then
      IsTargetAttribute = True
     End If
    Next
    If IsTargetAttribute = False Then
     Dim NextTestAttributeSet As New ArrayList
     For Each Attribute As Integer In TestAttributeSet
      NextTestAttributeSet.Add(Attribute)
     Next
     NextTestAttributeSet.Add(i)
     CheckTransitiveDependency(Attributes, FunctionalDependencies, NextTestAttributeSet, TargetAttributeSet, TransitiveDeterminants)
    End If
   Next
  End If
 End Function

#End Region

End Class
